//
// Charles Fried - Twitter: @pencil_stroke
//
// Color tracking
// Instructables
//


import processing.video.*;
Capture cam;

int threshold = 80; // Threshold use in color tracking, you can play around with it
char programState = 'A'; // The current step of the program
String weight = " "; // Weight of the sample
String name = " "; // Name of the sample

color leftColour = color(255, 0, 0); // Left target colour
color rightColour = color(255, 0, 0); // Right target colour

PVector leftMarker = new PVector(0, 0, 0); // Left start position
PVector rightMarker = new PVector(0, 0, 0); // Right start position

PVector leftStart = new PVector(0, 0, 0); // Left deflected position
PVector rightStart = new PVector(0, 0, 0); // Right deflected position

float rightDef = 0; // Right deflecteion
float leftDef = 0; // Left deflecteion
float deflection = 0; // Averaged deflection

Table results; // CSV file
int saveCount = 0; // Number of meansurements saved

void setup() {
  size(960, 480);
  rectMode(CENTER);
  textAlign(CENTER, CENTER);

  results = loadTable("results.csv");

  // Start Camerara
  String[] cameras = Capture.list();
  printArray(cameras);
  (cam = new Capture(this, cameras[1])).start();
  while (cam.width == 0)  delay(10);
}

void draw() {

  switch(programState) {

  case 'A': // User input sample name
    if (keyPressed && key == ENTER && name != " ") programState = 'B';
    background(255);
    fill(0);
    text("Enter Sample Name: " + name, width*0.5, height*0.5);
    break;

  case 'B': // User input sample weight

    if (keyPressed && key == ENTER && programState == 'B' && weight != " ") programState = 'C';
    background(255);
    text("Enter Sample Weight: " + weight, width*0.5, height*0.5);
    break;

  case 'C': // Set left marker color

    PImage img = cam.get();
    img.resize(0, height);
    image(img, 0, 0);

    if (keyPressed && key == ENTER && leftColour != color(255, 0, 0)) {
      leftStart = findMarker(cam, threshold, 0,int( width*0.35), leftColour); // save left start position
      programState = 'D';
    }

    cursor(CROSS);
    if (mousePressed) leftColour = get(mouseX, mouseY);

    fill(get(mouseX, mouseY));
    stroke(brightness(get(mouseX, mouseY))<127 ? 255 : 0);
    rect(mouseX+10, mouseY+10, 20, 20);

    stroke(0);
    line(int(width*0.35), 0, int(width*0.35), height);
    line(int(width*0.7), 0, int(width*0.7), height);

    noStroke();
    fill(255);
    rect(width*0.85, height/2, width*0.3, height);
    fill(255, 240);
    rect(width*0.525, height/2, width*0.35, height);
    fill(0);
    text("Select colour of left marker.", width*0.85, height*0.45);
    fill(leftColour);
    rect(width*0.85, height*0.55, 20, 20); 

    break;

  case 'D': // Set right marker color

    img = cam.get();
    img.resize(0, height);
    image(img, 0, 0);

    if (keyPressed && key == ENTER && rightColour != color(255, 0, 0)) {
      rightStart = findMarker(cam, threshold, int(width*0.35), int(width*0.7), rightColour); // save right start position
      programState = 'E';
    }

    if (mousePressed) rightColour = get(mouseX, mouseY);

    fill(get(mouseX, mouseY));
    stroke(brightness(get(mouseX, mouseY))<127 ? 255 : 0);
    rect(mouseX+10, mouseY+10, 20, 20);

    stroke(0);
    line(int(width*0.35), 0, int(width*0.35), height);
    line(int(width*0.7), 0, int(width*0.7), height);

    noStroke();
    fill(255);
    rect(width*0.85, height/2, width*0.3, height);
    fill(255, 240);
    rect(width*0.175, height/2, width*0.35, height);
    fill(0);
    text("Select colour of right marker.", width*0.85, height*0.45);
    fill(rightColour);
    rect(width*0.85, height*0.55, 20, 20); 

    break;

  case 'E': // Start testing
    cursor(HAND);
    deflection = (leftStart.dist(leftMarker)+rightStart.dist(rightMarker))/2;

    img = cam.get();
    img.resize(0, height);
    image(img, 0, 0);

    rectMode(CENTER);
    fill(255);
    rect(width*0.85, height/2, width*0.3, height);
    stroke(0);
    line(int(width*0.35), 0, int(width*0.35), height);
    line(int(width*0.7), 0, int(width*0.7), height);

    noStroke();
    fill(255, 0, 0);
    ellipse(leftStart.x, leftStart.y, 5, 5);
    ellipse(rightStart.x, rightStart.y, 5, 5);

    stroke(255, 0, 0);
    leftMarker = findMarker(cam, threshold, 0, int(width*0.35), leftColour);
    line(leftMarker.x, leftMarker.y, leftStart.x, leftStart.y);
    rightMarker = findMarker(cam, threshold, int(width*0.35), int(width*0.7), rightColour);
    line(rightMarker.x, rightMarker.y, rightStart.x, rightStart.y);
    drawTarget(leftMarker);
    drawTarget(rightMarker);

    if (leftStart.dist(leftMarker) > leftDef) leftDef = leftStart.dist(leftMarker);
    if (rightStart.dist(rightMarker) > rightDef) rightDef = rightStart.dist(rightMarker);

    // Display Data

    fill(0);
    text("Left target: ", width*0.85, height*0.67);
    text("Right target: ", width*0.85, height*0.71);
    text("Left deflection: " + leftDef, width*0.85, height*0.78);
    text("Right deflection: " + rightDef, width*0.85, height*0.82);
    fill(leftColour);
    rect(width*0.9, height*0.67, 10, 10);
    fill(rightColour);
    rect(width*0.9, height*0.71, 10, 10);

    displayForce(width*0.82, 300, deflection);

    if (button(width*0.89, height*0.92, 55, "RESET")) rightDef = leftDef = 0;
    button(width*0.81, height*0.92, 55, "WRITE");

    break;
  }
}

void captureEvent(Capture capture) {
  capture.read();
  redraw = true;
}